package multex; //Msg.java

import multex.Exc;
import java.lang.String;

//Change history:
//2001-04-12  Knabe  Static preallocation of a ahared DummyHereException object
//2000-12-13  Knabe  New methods get/print-Report report the message chain
//                   and the stack trace
//2000-12-13  Knabe  report-Methods renamed to get/print-Messages
//2000-10-25  Knabe  Reporting onto screen relocated into class multex.Awt
//2000-07-07  Knabe  Meldungstext holen+formatieren verlagert in Klasse MsgText
//2000-07-04  Knabe  report mit Varianten fr Print-Writer/Stream
//1999-03-23  Knabe  Parametrierung wie in Java-Tutorial/i18n/format
//1999-03-18  Knabe  erstellt

/** Services for reporting onto Streams and Strings the messages for any exception.
  The exception can be <UL>
    <LI> with parameters (subclass of Exc)
    <LI> with cause chain and parameters (subclass of Failure)
    <LI> or any standard Java exception (other subclass of java.lang.Throwable)
  </UL>

  @author Christoph Knabe, Berlin, Copyright 1999-2000
*/
public class Msg {


//static message parts:

/**The name of this class in the format package.class*/
private static final String _className = new Msg().getClass().getName();
private static final char _beep = '\u0007';


/**Reports i_throwable and its chained causing exceptions to System.err
  @see #printReport(StringBuffer,Throwable)
*/
public static void printReport(final Throwable i_throwable){
  printReport(System.err, i_throwable);
}//printReport

/**Reports i_throwable and its chained causing exceptions to io_destination
  @see #printReport(StringBuffer,Throwable)
*/
public static void printReport(
  final java.io.PrintStream io_destination, final Throwable i_throwable
){
  printReport(new java.io.PrintWriter(io_destination, true), i_throwable);
}//printReport

/**Reports i_throwable and its chained causing exceptions to io_destination
  @see #printReport(StringBuffer,Throwable)
*/
public static void printReport(
  final java.io.PrintWriter io_destination, final Throwable i_throwable
){
  final StringBuffer text = new StringBuffer();
  printReport(text, i_throwable);
  io_destination.println(text);
  io_destination.print(_beep);
  _checkPrintSuccess(io_destination, i_throwable, "REPORT");
}//printReport

/**Reports i_throwable and its chained causing exceptions to io_destination,
  detail description here.
  Reports the message chain of i_throwable by
  {@link #printMessages(StringBuffer,Throwable)}
  and the stack trace of i_throwable by
  {@link #printStackTrace(StringBuffer,Throwable)}.
  The report is appended to io_destination.
*/
public static void printReport(
  final StringBuffer io_destination, final Throwable i_throwable
){
  printMessages(io_destination, i_throwable);
  io_destination.append(Failure.lineSeparator);
  io_destination.append(stackTraceFollows);
  io_destination.append(Failure.lineSeparator);
  printStackTrace(io_destination, i_throwable);
}//printReport

/**The String used to separate the message chain of an exception from
  the stack trace of it.
*/
public static String stackTraceFollows
= "----------Stack Trace follows:----------";

/**Returns the report of i_throwable and its
  chained causing exceptions.
  @see #printReport(StringBuffer,Throwable)
*/
public static String getReport(final Throwable i_throwable){
  final StringBuffer result = new StringBuffer();
  printReport(result, i_throwable);
  return result.toString();
}//getReport

/**Reports the message texts of i_throwable and its
  chained causing exceptions to System.err
  @see #printMessages(StringBuffer,Throwable)
*/
public static void printMessages(final Throwable i_throwable){
  printMessages(System.err, i_throwable);
}//printMessages

/**Reports the message texts of i_throwable and its
  chained causing exceptions to io_destination
  @see #printMessages(StringBuffer,Throwable)
*/
public static void printMessages(
  final java.io.PrintStream io_destination, final Throwable i_throwable
){
  printMessages(new java.io.PrintWriter(io_destination, true), i_throwable);
}//printMessages

/**Reports the message texts of i_throwable and its
  chained causing exceptions to io_destination
  @see #printMessages(StringBuffer,Throwable)
*/
public static void printMessages(
  final java.io.PrintWriter io_destination, final Throwable i_throwable
){
  io_destination.print(_beep);
  final StringBuffer text = new StringBuffer();
  printMessages(text, i_throwable);
  io_destination.println(text);
  _checkPrintSuccess(io_destination, i_throwable, "MESSAGES");
}//printMessages

/**Checks if the last printing onto io_destination was successful.
  Reports an error message and the stack trace of i_throwable onto System.err,
  otherwise.
*/
private static void _checkPrintSuccess(
  final java.io.PrintWriter io_destination, final Throwable i_throwable,
  final String i_what
){
  if(io_destination.checkError()){
    System.err.println(_beep);
    System.err.print(_className);
    System.err.print(": COULD NOT PRINT ");
    System.err.print(i_what);
    System.err.print(" FOR:");
    System.err.println();
    i_throwable.printStackTrace();
  }
}

/**Returns the message texts of i_throwable and its
  chained causing exceptions.
  @see #printMessages(StringBuffer,Throwable)
*/
public static String getMessages(final Throwable i_throwable){
  final StringBuffer result = new StringBuffer();
  MsgText.appendMessageChain(result, i_throwable, Failure.lineSeparator);
  return result.toString();
}//getMessages

/**Reports the message texts of i_throwable and its
  chained causing exceptions, detail description here.
  Reports the paramerized textual message of i_throwable,
  or if this is not available, the  name and parameters of i_throwable.
  Reports the same for each of the eventually existing chain of causing
  Throwable exceptions. The report is appended to io_destination.

  The message texts for the exceptions are taken via
  {@link MsgText#appendMessageChain}.
  The concept of chaining causing exceptions is described in the class
  {@link Failure}.
*/
public static void printMessages(
  final StringBuffer io_destination, final Throwable i_throwable
){
  MsgText.appendMessageChain(io_destination, i_throwable, Failure.lineSeparator);
}//printMessages

/**Prints the compactified stack trace of i_throwable and its
  chained causing Throwable exceptions to System.err.
  @see #printStackTrace(StringBuffer,Throwable)
  */
public static void printStackTrace(final Throwable i_throwable){
  printStackTrace(System.err, i_throwable);
}//printStackTrace

/**Prints the compactified stack trace of i_throwable and its
  chained causing Throwable exceptions to io_destination.
  @see #printStackTrace(StringBuffer,Throwable)
  */
public static void printStackTrace(
  final java.io.PrintStream io_destination, final Throwable i_throwable
){
  printStackTrace(new java.io.PrintWriter(io_destination, true), i_throwable);
}//printStackTrace

/**Prints the compactified stack trace of i_throwable and its
  chained causing Throwable exceptions to io_destination.
  @see #printStackTrace(StringBuffer,Throwable)
*/
public static void printStackTrace(
  final java.io.PrintWriter io_destination, final Throwable i_throwable
){
  io_destination.print(getStackTrace(i_throwable));
  io_destination.flush();
}//printStackTrace

/**Returns the compactified stack trace of i_throwable and its
  chained causing Throwable exceptions.
  @see #printStackTrace(StringBuffer,Throwable)
*/
public static String getStackTrace(final Throwable i_throwable){
  final StringBuffer result = new StringBuffer();
  printStackTrace(result, i_throwable);
  return result.toString();
}//getStackTrace

/**Prints the compactified stack trace of i_throwable and its
  chained causing Throwable exceptions appending it to io_destination, detail description
  here. The contents is provided by the method
  {@link Failure#appendCompactStackTrace(StringBuffer,Throwable,Exception)}.
*/
public static void printStackTrace(
  final StringBuffer io_destination, final Throwable i_throwable
){
  _dummyHereException.fillInStackTrace();
  Failure.appendCompactStackTrace(io_destination, i_throwable, _dummyHereException);
}

/**User should not construct objects of this class*/
private Msg(){}

/**Dummy exception used to get the stack trace of "here", i.e. the place, where
  the method {@link #printStackTrace(StringBuffer, Throwable) } was invoked.
  ???still necessary?
*/
private static class _DummyHereException extends Exception {}

private static _DummyHereException _dummyHereException
= new _DummyHereException();

}//Msg